#!/usr/bin/env python
# coding: utf-8

# In[1]:


import hashlib
import os
import shutil
import subprocess
import uuid
from typing import Optional, List
import gzip
import json
from multiprocessing.pool import ThreadPool

import pandas as pd
import requests
from cooler import Cooler
from pydantic import BaseModel


# In[2]:


class HiCExperiment(BaseModel):
    """A class to represent a Hi-C experiment"""

    accession: Optional[str] = None
    assay_term_name: Optional[str] = None
    assay_title: Optional[str] = None
    award: Optional[dict] = None
    biosample: Optional[str] = None
    biosample_ontology: Optional[dict] = None
    biosample_summary: Optional[str] = None
    dbxrefs: Optional[list] = None
    description: Optional[str] = None
    lab: Optional[dict] = None
    replicates: Optional[list] = None
    status: Optional[str] = None
    hic_accession: Optional[str] = None

    def __repr__(self):
        return f"{self.biosample_summary} - {self.accession}: {self.hic_accession}"


class HiCExperimentList(BaseModel):
    experiments: List[HiCExperiment]


def download_encode(accession, extension):
    """Download an ENCODE file by its accession and store the result on disk"""
    url = f"https://www.encodeproject.org/files/{accession}/@@download/{accession}.{extension}"
    local_filename = f"/net/common/cache/encode/GRCh38/{accession}.{extension}"
    if os.path.exists(local_filename):
        return local_filename
    with requests.get(url, stream=True, allow_redirects=True) as r:
        temp_file = str(uuid.uuid4()) + f".{extension}"
        print(f"Downloading {accession} to {temp_file}")
        with open(temp_file, "wb") as f:
            shutil.copyfileobj(r.raw, f)
        shutil.move(temp_file, local_filename)
    return local_filename


def verify_encode_file(accession, extension):
    """Verify md5 checksum of encode file"""
    url = f"https://www.encodeproject.org/files/{accession}/?format=json"
    response = requests.get(url)
    payload = response.json()
    md5sum = payload["md5sum"]
    local_filename = download_encode(accession, extension)

    # Get md5sum of file
    with open(local_filename, "rb") as f:
        file_hash = hashlib.md5()
        while chunk := f.read(67108864):  # 64 MB chunks
            file_hash.update(chunk)
        file_md5sum = file_hash.hexdigest()

    # Check md5sum matches
    if md5sum != file_md5sum:
        os.remove(local_filename)
        print(f"MD5 checksums don't match for {accession}!")
        raise Exception(f"MD5 checksums don't match for {accession}!")
    print(f"Verified MD5 checksum for {accession}.{extension}")
    return local_filename


def hic_to_mcool(accession):
    """Convert a hic file to a mcool file"""
    mcool_file = f"{accession}.mcool"
    if os.path.exists(mcool_file):
        try:
            clr = Cooler(f"{mcool_file}::/resolutions/50000")
            print(clr.info)
            return mcool_file
        except OSError as e:
            os.remove(mcool_file)
            raise e

    hic_file = verify_encode_file(accession, "hic")
    temp_name = str(uuid.uuid4())
    temp_hic = f"{temp_name}.hic"
    temp_mcool = f"{temp_name}.mcool"
    shutil.copy(hic_file, temp_hic)
    subprocess.run(["/home/reimonnt/miniconda3/envs/hictk/bin/hictk", "convert", temp_hic, temp_mcool], capture_output=False)
    shutil.move(temp_mcool, mcool_file)
    os.remove(temp_hic)
    return mcool_file


# In[3]:


primary_cell = pd.read_csv("metadata-primary-cell-samples.csv")
tissues = pd.read_csv("metadata-tissue-samples.csv")


# In[4]:


metadata = pd.concat([primary_cell, tissues])
metadata.head()


# In[5]:


accessions = metadata.hic_accession.to_list()
print(f"Number of accessions: {len(accessions)}")


# In[6]:


# accessions


# In[ ]:


# Download files
with ThreadPool(16) as pool:
    mcool_filelist = pool.map(hic_to_mcool, accessions)


# In[ ]:




